/* GSequencer - Advanced GTK Sequencer
 * Copyright (C) 2005-2024 Joël Krähemann
 *
 * This file is part of GSequencer.
 *
 * GSequencer is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * GSequencer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GSequencer.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __AGS_CROP_NOTE_POPOVER_H__
#define __AGS_CROP_NOTE_POPOVER_H__

#include <glib.h>
#include <glib-object.h>

#include <gtk/gtk.h>

#include <ags/libags.h>
#include <ags/libags-audio.h>
#include <ags/libags-gui.h>

G_BEGIN_DECLS

#define AGS_TYPE_CROP_NOTE_POPOVER                (ags_crop_note_popover_get_type())
#define AGS_CROP_NOTE_POPOVER(obj)                (G_TYPE_CHECK_INSTANCE_CAST((obj), AGS_TYPE_CROP_NOTE_POPOVER, AgsCropNotePopover))
#define AGS_CROP_NOTE_POPOVER_CLASS(class)        (G_TYPE_CHECK_CLASS_CAST((class), AGS_TYPE_CROP_NOTE_POPOVER, AgsCropNotePopoverClass))
#define AGS_IS_CROP_NOTE_POPOVER(obj)             (G_TYPE_CHECK_INSTANCE_TYPE ((obj), AGS_TYPE_CROP_NOTE_POPOVER))
#define AGS_IS_CROP_NOTE_POPOVER_CLASS(class)     (G_TYPE_CHECK_CLASS_TYPE ((class), AGS_TYPE_CROP_NOTE_POPOVER))
#define AGS_CROP_NOTE_POPOVER_GET_CLASS(obj)      (G_TYPE_INSTANCE_GET_CLASS (obj, AGS_TYPE_CROP_NOTE_POPOVER, AgsCropNotePopoverClass))

#define AGS_CROP_NOTE_POPOVER_MAX_WIDTH (128 * 16)

typedef struct _AgsCropNotePopover AgsCropNotePopover;
typedef struct _AgsCropNotePopoverClass AgsCropNotePopoverClass;

struct _AgsCropNotePopover
{
  GtkPopover popover;

  guint connectable_flags;

  GtkCheckButton *absolute;
  
  GtkCheckButton *in_place;
  GtkCheckButton *do_resize;

  GtkSpinButton *crop_note;
  GtkSpinButton *padding_note;

  GtkBox *action_area;

  GtkButton *activate_button;  
};

struct _AgsCropNotePopoverClass
{
  GtkPopoverClass popover;
};

GType ags_crop_note_popover_get_type(void);

AgsCropNotePopover* ags_crop_note_popover_new();

G_END_DECLS

#endif /*__AGS_CROP_NOTE_POPOVER_H__*/
